/*
 * gpoll v0.0.1
 *
 *
 * This work has been made possible in part by a grant from the Cisco
 * University Research Program Fund at Community Foundation Silicon Valley.
 *
 * This software is copyright (c) 2007 Swinburne University
 * of Technology, Melbourne, Australia (CRICOS number 00111D).
 * This software was developed by Warren Harrop (wazz@swin.edu.au)
 * while a PhD student at the Centre for Advanced Internet
 * Architectures (http://caia.swin.edu.au).
 *
 * This file is part of lcmon_v1.0
 *
 * gpoll is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * gpoll is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * Seriously, it's version 0.0.1.
 *
 * You should have received a copy of the GNU General Public License
 * along with fakematter; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * \author Carl Javier - CAIA, Swinburne University of Technology
 * \author Lucas Parry - CAIA, Swinburne University of Technology
 * 
 * Developed during Winter Internship at CAIA, Swinburne Unversity of Technology
 * June 18 - August 3rd 2006
 */

#include "gpoll.h"

#define MAX_SCALE 25
#define MAX_CPU 100
#define MAX_BOUNCE 60


#define RED "ff0000"
#define GREEN "2ff425"
#define BLUE "0000ff"
#define ORANGE "ff6000"

#define GREP "/usr/bin/grep"
#define CUT "/usr/bin/cut"

int m1,m2,m3,m4,m5,m6,zero,runtime=-1;
int delay=0;

struct greynet_host *greynet_hosts; // will point to an array of greynet hosts 

int numOfGreynetHosts = 0;

int PPSwindow = 5;


/**
 * Prints a usage message.
 */
void usage(void)
{
	//while ((ch = getopt(argc, argv, "123456zvc:a:p:t:h")) != -1)
   fprintf(stderr,
	"gpoll [-v] [-h] -c <config file> -a <ip address> [-p <port>]\n"
	"\t-v Verbose debug\n"
	"\t-c <config file>\n"
	"\t-a IP Address of the L3DGEWorld server\n"
	"\t-p Port that the L3DGEWorld server is listening on\n"
	"\t-z write out zeros for all metrics. commonly used with -t0 and no other metrics\n"
	"\t-h This help message\n");
}

/**
 * finds the number of valid host lines in the specifed config file.
 * \param configfileloc is the path to the config file
 * \return the number of hosts
 */
int findNumOfConfiguredHosts(char * configfileloc)
{

  int numOfGreynetHosts=0;
  FILE* configfile;
  char string[1024];
  char IPtemp[1024];
  int IDtemp;

  if ( (configfile = fopen( configfileloc,"r")) == NULL)
  {
    printf("Can't open file: %s\nCan't continue without this. Pulling the chute!\n", configfileloc);
    exit(2);
  }
  while(!(fgets(string,  1024, configfile) == NULL))
  {
    //if the first char of the line is '#' or '[' or '\n' or ' ' ignore: else, read it in
    if (! (string[0] == '#' || string[0] == '[' || string[0] == '\n' || string[0] == ' ' || string[0] == '\t'))
    {	
      if( sscanf(string,"%s %d", IPtemp, &IDtemp) != 2)
      {
        printf("Bad config file line?: %s...\n",string);
      }
      else
      {
        numOfGreynetHosts++;
      }

    }

  }
  return numOfGreynetHosts;
}

/**
 * reads in a config file and sets up our greynet_hosts array.
 * \param configfileloc is the path to the config file
 */
void readConfigFile(char * configfileloc)
{
  FILE* configfile;
  char string[1024];
  char IPtemp[1024];
  struct in_addr tempIP;
  int IDtemp;

  numOfGreynetHosts=0;
  if ( (configfile = fopen( configfileloc,"r")) == NULL)
  {
    printf("Can't open file: %s\nCan't continue without this. Pulling the chute!\n", configfileloc);
    exit(2);
  }
  while(!(fgets(string,  1024, configfile) == NULL))
  {
    //if the first char of the line is '#' or '[' or '\n' or ' ' ignore: else, read it in
    if (! (string[0] == '#' || string[0] == '[' || string[0] == '\n' || string[0] == ' ' || string[0] == '\t'))
    {	
      debug_printf("Config file line command: %s",string);

      if( sscanf(string,"%s %d", IPtemp, &IDtemp) != 2)
      {
        printf("Bad config file line?: %s...\n",string);
      }
      else
      {
        // put entry into array
        tempIP.s_addr =inet_addr( IPtemp);
        memcpy( &greynet_hosts[numOfGreynetHosts].ip_address , &tempIP, sizeof(tempIP) );
        greynet_hosts[numOfGreynetHosts].hostid = IDtemp;

        debug_printf("ip: %s, hostid: %03i, array position: %i.\n\n",IPtemp,IDtemp,numOfGreynetHosts);
        numOfGreynetHosts++;
      }

    }

  }
}

/**
* Send variable names for each metric. These variable names are metric names that will be displayed in front of 
* an entity when clients fly towards it. 
* There are 6 metrics to set. Not all metrics are displayed. 
*
*/
void sendCVARNames (void)
{

		sendMessage(0,METRIC_ONE,FIELD_NAME,"CPU Usage"); // Rate of spinning object represents CPU load percetage (0 - 100%)
		sendMessage(0,METRIC_TWO,FIELD_NAME,"PPS"); //Bounce height represents the rate of Packets Per Second (combined in and out).
		sendMessage(0,METRIC_THREE,FIELD_NAME,"Memory Usage"); //Scale of an object represents
		//sendMessage(0,METRIC_FOUR,FIELD_NAME,"PPS"); //Roll. Entity is rolled 180 degrees
		//sendMessage(0,METRIC_FIVE,FIELD_NAME,"PPS"); //RGB colour, usually not displayed
		//sendMessage(0,METRIC_SIX,FIELD_NAME,"PPS"); // Bounce Rate
}


//used to clear all metrics on the server
void zeroMetrics(void )
{
  int i;
  for (i=0;i<numOfGreynetHosts;i++)
  {
    sendMessage(greynet_hosts[i].hostid,METRIC_ONE,FIELD_VALUE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_ONE,FIELD_RATE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_TWO,FIELD_VALUE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_TWO,FIELD_RATE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_THREE,FIELD_VALUE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_THREE,FIELD_RATE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_FOUR,FIELD_VALUE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_FOUR,FIELD_RATE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_FIVE,FIELD_VALUE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_FIVE,FIELD_RATE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_SIX,FIELD_VALUE,"0");
    sendMessage(greynet_hosts[i].hostid,METRIC_SIX,FIELD_RATE,"0");
    usleep(120000);
  }
}




/**
* writeSupercomputer initiates the telnet command to retrieve data about the supercomputer cluster nodes. 
* An XML file is retrieved from one of the hosts and written to a file.
* Information from the XML file is extracted and sent to the server using functions from 'l3dgecomms.c'.
* 
* The config file specified must be the correct IP address correlating to the one being used by the LCMON server which uses
* hostnames instead of IP addresses. It is used to map each cluster node to a host id. 
*
*/
void writeSupercomputer(void)
{


	//Determine full path directory
	FILE *fp_pwd;
	fp_pwd=popen("pwd","r");
	
	char pwd[1024];
	fgets(pwd,1024,fp_pwd);
	fclose(fp_pwd);
	
	pwd[strlen(pwd)-1]='\0'; //Replace \n from end of string
	debug_printf("\nWorking Directory:%s\t",pwd);

	//Fetch Cluster-node s
 	//char telnet_cmd[1024];
 	//debug_printf("Executing CM: telnet shrek04.ssi.swin.edu.au 8649 > %s/greenmachine.XML",pwd);
 	//sprintf(telnet_cmd,"/usr/bin/telnet shrek04.ssi.swin.edu.au 8649 > %s/greenmachine.XML",pwd);
 	
 	//system(telnet_cmd);
 	//sleep(3);
	

	//Grep through xml file
	char get_hostnames[1024];
	sprintf(get_hostnames,"%s \"HOST NAME=\" %s/greenmachine.XML | %s -d \" \" -f 2 | %s -d '\"' -f 2",GREP,pwd,CUT,CUT);
	char get_ips[1024];
	sprintf(get_ips,"grep \"HOST NAME=\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);
	char get_last_unix_time[1024];
	sprintf(get_last_unix_time,"grep \"HOST NAME=\" %s/greenmachine.XML | cut -d \" \" -f 4 | cut -d '\"' -f 2",pwd);
	char get_cpu_speed[1024];
	sprintf(get_cpu_speed,"grep \"cpu_speed\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);
	//char get_os_name[]="grep \"os_name\" testout3 | cut -d \" \" -f 3 | cut -d '\"' -f 2";
	char get_diskspace_total[1024];
	sprintf(get_diskspace_total,"grep \"disk_total\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);
	char get_diskspace_free[1024];
	sprintf(get_diskspace_free,"grep \"disk_free\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);
	char get_unix_uptime[1024];
	sprintf(get_unix_uptime,"grep \"HOST NAME=\" %s/greenmachine.XML | cut -d \" \" -f 9 | cut -d '\"' -f 2",pwd);

	//Obtain metrics for visual representation for each cluster node
	char get_cpu_idle[1024];
	sprintf(get_cpu_idle,"grep \"cpu_idle\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);
	char get_mem_free[1024];
	sprintf(get_mem_free,"grep \"mem_free\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);
	char get_mem_total[1024];
	sprintf(get_mem_total,"grep \"mem_total\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);
	char get_pkts_in[1024];
	sprintf(get_pkts_in,"grep \"pkts_in\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);
	char get_pkts_out[1024];
	sprintf(get_pkts_out,"grep \"pkts_out\" %s/greenmachine.XML | cut -d \" \" -f 3 | cut -d '\"' -f 2",pwd);


//	FILE *fp;
	FILE *fp_hostnames;
	FILE *fp_cpu_idle;
	FILE *fp_mem_free;
	FILE *fp_mem_total;
	FILE *fp_pkts_in;
	FILE *fp_pkts_out;
	FILE *fp_ips;
	FILE *fp_last_unix_time,*fp_cpu_speed,*fp_diskspace_total,*fp_diskspace_free,*fp_unix_uptime;//*fp_os_name // all clusters run Linux


	//Number of nodes in the XML file
	int num_of_hosts=numOfGreynetHosts;
	
	//Setup metric arrays
	char hostnames[num_of_hosts][1024];
	char cpu_idle[num_of_hosts][1024];
	char mem_free[num_of_hosts][1024];
	char mem_total[num_of_hosts][1024];
	char pkts_in[num_of_hosts][1024];
	char pkts_out[num_of_hosts][1024];
	char ips[num_of_hosts][1024];

	char last_unix_time[num_of_hosts][1024];
	char cpu_speed[num_of_hosts][1024];
	//char os_name[num_of_hosts][1024];
	char diskspace_total[num_of_hosts][1024];
	char diskspace_free[num_of_hosts][1024];
	char unix_uptime[num_of_hosts][1024];

	//Open Pipes to shell grep cmd
	fp_hostnames = popen(get_hostnames,"r");
	fp_cpu_idle = popen(get_cpu_idle,"r");
	fp_mem_free = popen(get_mem_free,"r");
	fp_mem_total = popen(get_mem_total,"r");
	fp_pkts_in = popen(get_pkts_in,"r");
	fp_pkts_out = popen(get_pkts_out,"r");
	fp_ips = popen(get_ips,"r");
	
	fp_last_unix_time = popen(get_last_unix_time,"r");
	fp_cpu_speed = popen(get_cpu_speed,"r");
	fp_diskspace_total = popen(get_diskspace_total,"r");
	fp_diskspace_free = popen(get_diskspace_free,"r");
	fp_unix_uptime = popen(get_unix_uptime,"r");
	//fp_os_name = popen(get_os_name,"r");


	
	//Fill Metric Arrays
	int counter=0;
	for (counter=0; counter <num_of_hosts;counter++)
	{
	
		if(fscanf(fp_hostnames,"%s",hostnames[counter])==EOF)
			break;
		fscanf(fp_cpu_idle,"%s",cpu_idle[counter]);
		fscanf(fp_mem_free,"%s",mem_free[counter]);
		fscanf(fp_mem_total,"%s",mem_total[counter]);
		fscanf(fp_pkts_in,"%s",pkts_in[counter]);
		fscanf(fp_pkts_out,"%s",pkts_out[counter]);
		fscanf(fp_ips,"%s",ips[counter]);

		fscanf(fp_last_unix_time,"%s",last_unix_time[counter]);
		fscanf(fp_cpu_speed,"%s",cpu_speed[counter]);
		fscanf(fp_diskspace_total,"%s",diskspace_total[counter]);
		fscanf(fp_diskspace_free,"%s",diskspace_free[counter]);
		fscanf(fp_unix_uptime,"%s",unix_uptime[counter]);
		//fscanf(fp_os_name,"%s",os_name[counter]);
	}

	debug_printf("\n\nHOSTNAME: %s\n\n",hostnames[1]);
	
	fclose(fp_hostnames);
	fclose(fp_cpu_idle);
	fclose(fp_mem_free);
	fclose(fp_mem_total);
	fclose(fp_pkts_in);
	fclose(fp_pkts_out);
	fclose(fp_ips);

	fclose(fp_last_unix_time);
	fclose(fp_cpu_speed);
	fclose(fp_diskspace_total);
	fclose(fp_diskspace_free);
	fclose(fp_unix_uptime);
	//fclose(fp_os_name);


	//Send stats to l3dgeworld server
	int sleep_time=50000; //flooding the server can cause network interuptions especially when sending to localhost.
	int i=0;
	
	//Send display metric names of each entity. 
	sendCVARNames();
	

	//Process metrics and send to LCMON server
	for (i=0; i < num_of_hosts; i++)
	{
		//First need to map the host id to the first cluster node metrics found in the XML file
		int hostid_tmp=0;
		

		//Determine what hostid the LCMON server has assigned the first cluster node in the XML file
		int counter=0;
		for (counter=0; counter <num_of_hosts;counter++)
		{
			if (greynet_hosts[counter].ip_address.s_addr == inet_addr(ips[i]))
			{
				hostid_tmp=greynet_hosts[counter-1].hostid;
				break;
			}
		}
		debug_printf("\nWriting for cluster %s with host id : %i\n",hostnames[i],hostid_tmp);
		char cpuString[1024];
		float tempFloat;
		tempFloat=atof(cpu_idle[i]);
		
		float cpu_usage;
		cpu_usage=100-tempFloat;
		sprintf(cpuString,"%.i/100",(int)cpu_usage);
		
		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_ONE,FIELD_VALUE,cpuString);
		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_ONE,FIELD_RATE,cpuString);
		debug_printf("\nwriting CPU %i out of %i\n",hostid_tmp,num_of_hosts);
		usleep(sleep_time);
		
    		char colour[1024];
		char cvalue[1024];
		
		if (cpu_usage > 75)
		{
			sprintf(cvalue,"%s",RED);//Colour RED	
		}
		else if (cpu_usage > 50 && cpu_usage < 74 )
		{
			sprintf(cvalue,"%s",ORANGE);//Colour Orange
		}
		else if (cpu_usage > 25 && cpu_usage < 49 )
		{
			sprintf(cvalue,"%s",GREEN);//Colour Green
		}
		else
		{
			sprintf(cvalue,"%s",BLUE);//Colour Blue
		}
		
		
		sprintf(colour,"%s",cvalue);
		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_FIVE,FIELD_VALUE,colour);
    		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_FIVE,FIELD_RATE,colour);

	//Write Memory usage(mem_total - mem_free)
	//Memory scaled as a percetaged and multiplied by MAX_SCALE

		char memString[1024];
		float mem_total_float;
		float mem_free_float;
		float mem_used_float;
		mem_total_float=atof(mem_total[i]);
		mem_free_float=atof(mem_free[i]);
		mem_used_float=mem_total_float-mem_free_float;

		float temp;
		temp=(mem_used_float/mem_total_float) * MAX_SCALE;
		char Scale[1024];
		
		sprintf(memString,"%i/100",(int)((mem_used_float/mem_total_float)*100));
		sprintf(Scale,"%lf",temp);
		
		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_THREE,FIELD_VALUE,memString);
		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_THREE,FIELD_RATE,Scale);
		
		usleep(sleep_time);


// // 	//Write Network Traffic (Packets Per Second coming into the cluster node)
// // 	//Network traffic is scaled logorithmatically

		char String[1024];
		//float tempFloat;
		//tempFloat=atof(pkts_in[i]);
		//printf("\n%f",tempFloat);
		float pkt_rate=atof(pkts_in[i]);
		int pkt_rate_scaled;
		pkt_rate_scaled=0;
		
		if (pkt_rate>1200)
		{
			pkt_rate_scaled=MAX_BOUNCE;
		}
		else if(pkt_rate >500 && pkt_rate<1199)
		{
			pkt_rate_scaled=30;
		}
		else if(pkt_rate >100 && pkt_rate<499)
		{
			pkt_rate_scaled=10;
		}
		else if(pkt_rate >50 && pkt_rate<99)
		{
			pkt_rate_scaled=4;
		}
		else
		{
			pkt_rate_scaled=0;
		}

		sprintf(String,"%i",pkt_rate_scaled);
		//set bounce height of how high an entity jumps
		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_TWO,FIELD_VALUE,pkts_in[i]);
		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_TWO,FIELD_RATE,String);

		usleep(sleep_time);

		//Set rate of how fast entity jumps //not used in this version of gpoll
		//sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_SIX,FIELD_VALUE,String);
		//sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_SIX,FIELD_RATE,String);


// 	//Send window information about each cluster node.

		char msgbuffer[4096];
		char line[512];
		strcpy(msgbuffer,"");

	//calculate Unix times
		debug_printf("\n\nCONVERT UNIX TIME\n");

		long epoch=atol(last_unix_time[i]);
		
		
		struct tm *t;
		t=gmtime((time_t *)&epoch );

		char date[1024];
		sprintf(date,"%s",asctime(t));
		
		struct tm *t2;
		long epochtwo=atol(unix_uptime[i]);
		t2=gmtime((time_t *)&epochtwo);
		char date2[1024];
		
		sprintf(date2,"%s",asctime(t2));

 		date[strlen(date)-1]='\0'; //Replace \n from end of string
		date2[strlen(date2)-1]='\0';

		debug_printf("Date1: %s\n",date);
		debug_printf("Date2: %s\n",date2);


		//Send IP address as second header in blue
		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_INFO,FIELD_NAME,ips[i]);
		
		usleep(sleep_time);
		
		sprintf(line, "@CPU: %sMHz  USAGE:%i/100 @", cpu_speed[i],(int)(100-atof(cpu_idle[i])));
        	strcat(msgbuffer,line);
		
		float mem_total_float_tmp;
		float mem_free_float_tmp;
		float mem_used_float_tmp;
		mem_total_float_tmp=atof(mem_total[i]);
		mem_free_float_tmp=atof(mem_free[i]);
		mem_used_float_tmp=mem_total_float_tmp-mem_free_float_tmp;

		
		sprintf(line, "Mem:%i/100(used/total)@", (int)((mem_used_float_tmp/mem_total_float_tmp)*100));
        	strcat(msgbuffer,line);
        	sprintf(line, "Disk:%s/%sGB(free/total)@",diskspace_free[i],diskspace_total[i]);
        	strcat(msgbuffer,line);
		sprintf(line, "PPS:%s(in)/%s(out)@",pkts_in[i],pkts_out[i]);
        	strcat(msgbuffer,line);
          	sprintf(line, "Reported:%s @", date);
         	strcat(msgbuffer,line);
         	sprintf(line, "Up Since: %s @", date2 );
         	strcat(msgbuffer,line);
		debug_printf("\nSTRING LENGTH: %i\n",strlen(msgbuffer));
      		sendMessage(greynet_hosts[hostid_tmp].hostid,METRIC_INFO,FIELD_VALUE,msgbuffer);
		//printf("\nPort%i,",i);
		usleep(120000);

	}
}

/**
 * main function.
 * \param argc is the number of arguments from the command line
 * \param argv[] is an array of strings containing the arguments
 * \return the exit status of the program
 */
int main(int argc, char *argv[])
{
  //int rc1;	
	//char *dev = NULL;
	//bpf_u_int32 mask;
	//bpf_u_int32 net;
	//int setup =0;
	//char *filterString = NULL;
	//char string[1024]; //yes, yes, mother i know...
	//char *tcpdumpfileloc;
	char *configfileloc = NULL;
	char *ipaddress = NULL;
	char *port = NULL;

	//command line vars
	int ch;
  
  srand(time(NULL));

	while ((ch = getopt(argc, argv, "123456zvc:d:a:p:t:h")) != -1)
	switch(ch) {
	case '1':
		m1++;
		break;
	case '2':
		m2++;
		break;
	case '3':
		m3++;
		break;
	case '4':
		m4++;
		break;
	case '5':
		m5++;
		break;
	case '6':
		m6++;
		break;
	case 'z':
		zero++;
		break;
	case 'd':
		delay = atoi(optarg);
		break;
	case 't':
		runtime = atoi(optarg);
		break;
	case 'v':
		debug++;
		break;
	case 'c':
		configfileloc = optarg;
		break;
	case 'a':
		ipaddress = optarg;
		break;
	case 'p':
		port = optarg;
		break;
	case 'h':
		break;
	case '?':
		printf("Unrecognised option in there somewhere...\n");
	default:
		usage();
		exit(1);
	}
	
	argc -= optind;
	argv += optind;

	//check some inital conditions and make sure all is good:
	if (!configfileloc)
	{
		printf("Error: No config file set.\nPulling the chute!\n");
		usage();
		exit(1);
	}

	if (ipaddress == NULL)
	{
		printf("Error: No ip address set.\nPulling the chute!\n");
		usage();
		exit(1);
	}

  // malloc us some memory to store stuff in
  greynet_hosts = malloc(findNumOfConfiguredHosts(configfileloc)*sizeof(struct greynet_host));
  if (greynet_hosts == NULL)
  {
    fprintf(stderr,"Couldn't allocate enough memory for the cluster node host array. exiting...\n");
    exit(EXIT_FAILURE);
  }
  
	//read in ye' olde' config file into our local array of greynet structs
  readConfigFile(configfileloc);

  debug_printf("setting up udp port");
  if (port) {
    setupCommsWithPort(ipaddress,atoi(port)); 
  }else{
    setupComms(ipaddress);
  }


	 if(zero)
	{	
		zeroMetrics();
	}
	else
	{
		writeSupercomputer();
		
	}

  closeComms();
  free(greynet_hosts);
return 0;
}


