/*
 * l3dge comms module 
 *
 *
 * This work has been made possible in part by a grant from the Cisco
 * University Research Program Fund at Community Foundation Silicon Valley.
 *
 * This software is copyright (c) 2007 Swinburne University
 * of Technology, Melbourne, Australia (CRICOS number 00111D).
 * This software was developed by Warren Harrop (wazz@swin.edu.au)
 * while a PhD student at the Centre for Advanced Internet
 * Architectures (http://caia.swin.edu.au).
 *
 * This file is part of l3dgeworld_v1.0
 *
 * fakematter is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * fakematter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * Seriously, it's version 0.0.1.
 *
 * You should have received a copy of the GNU General Public License
 * along with fakematter; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * \author Lucas Parry - CAIA, Swinburne University of Technology
 */

#include "l3dgecomms.h"
#define MSEC (1000)
struct sockaddr_in local,remote;
int L3DGESocket;
char  L3DGEToken[17];

void requestToken()
{
  char msg[1024];
  char *split;
  unsigned int size;
  //  char pps[32];
  
  // 0xffffffff lets q3a know that we're sending connectionless packets
  msg[0]=msg[1]=msg[2]=msg[3]=0xff;

  // specify l3dge messaging protocol, command cvarset
  strcpy(&(msg[4]),"gettoken"); 

  debug_printf("sent message of length %04i: %s\n", (int)strlen(msg),msg);
  if (sendto(L3DGESocket, msg, strlen(msg)+1, 0, (struct sockaddr *)&remote, sizeof(remote))<0)
  {
    printf("error sending! aborting...\n");
    exit(EXIT_FAILURE);
  }

  {
    int i;
    for (i=0;i<1024;i++)
    {
      msg[i]='\0';
    }
  }
  size=sizeof(remote);
  recvfrom(L3DGESocket, msg, 1024, 0, (struct sockaddr *)&remote, &size);
  split= memchr(&msg[4],(int)':',1024);
  if (split== NULL)
  {
    printf("error parsing token\n");
    exit(EXIT_FAILURE);

  }
  split++;
  strcpy(L3DGEToken,split);
  debug_printf("token recieved: %s\n",L3DGEToken);
}
// function decl (used only in this file)
int lookup_host(struct sockaddr_in *ra, char *hn, unsigned short rp);

/**
 * sendMessage  sends out the given data to the l3dgeworld server in the
 * correct syntax over a udp port
 * \param hostid is the id of the host who this data relates to. 1-64 for
 * the L3DGEWorld 2.0 map.
 * \param metric is the metric to update.
 * \param field is the field of the metric to update.
 * \param value is a string containing the data we wish to set.
 */
void sendMessage(int hostid, int metric, char *field, char *value)
{
  char msg[1024];
  //  char pps[32];
  char delimiter[2];
  
  // define our delimiter
  delimiter[0]=126; // tilde ~
  delimiter[1]='\0';

  // 0xffffffff lets q3a know that we're sending connectionless packets
  msg[0]=msg[1]=msg[2]=msg[3]=0xff;

  // specify l3dge messaging protocol, command cvarset
  sprintf(&msg[4],"l3dge %s %s%03i%s%d%s%s%s%s%s",
      L3DGEToken,delimiter,
      hostid,delimiter,
      metric,delimiter,
      field,delimiter,
      value,delimiter);

  debug_printf("sent message of length %04i: %s\n", (int)strlen(msg),msg);
  if (sendto(L3DGESocket, msg, strlen(msg)+1, 0, (struct sockaddr *)&remote, sizeof(remote))<0)
  {
    printf("error sending! aborting...\n");
    exit(EXIT_FAILURE);
  }
  usleep(10*MSEC);
}

/**
 * setupComms sets up the udp socket we need to send data to the l3dgeworld server 
 * \param ipAddress is the ip address of the l3dgeserver as a string in
 * standard dotted notation.
 */
void setupComms(char *ipAddress)
{
  setupCommsWithPort(ipAddress,27960); //default q3a port 
  
}

/**
 * setupCommsWithPort sets up the udp socket we need to send data to the l3dgeworld server when using a non-standard port
 * \param ipAddress is the ip address of the l3dgeserver as a string in.
 * standard dotted notation.
 * \param port is the port the l3dgeserver is listening on. 
 */
void setupCommsWithPort(char *ipAddress,int port)
{
  
  debug_printf("setting up udp port");
  L3DGESocket = socket(AF_INET, SOCK_DGRAM, 0);
  local.sin_family = AF_INET;
  local.sin_port = INADDR_ANY;;
  local.sin_addr.s_addr = INADDR_ANY;

  bind(L3DGESocket, (struct sockaddr *)&local, sizeof(local));

  if(!lookup_host(&remote, ipAddress, port))
  {
    exit(EXIT_FAILURE);
  }
  requestToken();
  usleep(10*MSEC);
}

/*
 * resolve a hostname to a sockaddr_in struct.
 * we first try treating it like an ip address in a.b.c.d notation
 * then, if that fails, we try to resolve using DNS ways
 * if all fails, we return 0. (failed)
 * if we get the sockaddr_in struct all filled out, we return 1.
 * from : http://attrition.org/tools/other/binfo.c
 */
int lookup_host( struct sockaddr_in *remoteAddress, char *hn, unsigned short remotePort)
{
   struct hostent *he;

   remoteAddress->sin_family = AF_INET;
   remoteAddress->sin_port = htons(remotePort);
   if ((remoteAddress->sin_addr.s_addr = inet_addr(hn)) != -1)
   {
     return 1;
   }
   if ((he = gethostbyname(hn)) != (struct hostent *)NULL)
   {
     memcpy(&remoteAddress->sin_addr.s_addr, he->h_addr, 4);
     return 1;
   }
   herror("Unable to resolve hostname\n");
   return 0;
}

/**
 * closeComms shuts down the udp socket
 */
void closeComms(void)
{
  debug_printf("tearing down udp port\n");
  close(L3DGESocket);
}
